@@ -1,9 +1,12 @@
+#[cfg(test)]
+#[path = "../../tests/unit/commands/generate_test.rs"]
+mod generate_test;
+
 use super::*;
 use std::io::BufReader;
-use std::process;
 use vrp_cli::extensions::generate::generate_problem;
 use vrp_cli::extensions::import::serialize_hre_problem;
-use vrp_pragmatic::format::problem::serialize_problem;
+use vrp_pragmatic::format::problem::{serialize_problem, Problem};
 use vrp_pragmatic::format::FormatError;
 use vrp_pragmatic::validation::ValidationContext;
 
@@ -76,48 +79,48 @@ pub fn get_generate_app<'a, 'b>() -> App<'a, 'b> {
         )
 }
 
-pub fn run_generate(matches: &ArgMatches) {
-    let input_format = matches.value_of(FORMAT_ARG_NAME).unwrap();
-    let input_files = matches
-        .values_of(PROTOTYPES_ARG_NAME)
-        .map(|paths: Values| paths.map(|path| BufReader::new(open_file(path, "input"))).collect::<Vec<_>>());
-    let locations_file = matches.value_of(LOCATIONS_ARG_NAME).map(|path| BufReader::new(open_file(path, "locations")));
-
-    let jobs_size = parse_int_value::<usize>(matches, JOBS_SIZE_ARG_NAME, "jobs size").unwrap();
-    let vehicles_size = parse_int_value::<usize>(matches, VEHICLES_SIZE_ARG_NAME, "vehicles size").unwrap();
-    let area_size = parse_float_value::<f64>(matches, AREA_SIZE_ARG_NAME, "area size");
-
-    match generate_problem(input_format, input_files, locations_file, jobs_size, vehicles_size, area_size) {
-        Ok(problem) => {
-            if let Err(errors) = ValidationContext::new(&problem, None).validate() {
-                eprintln!(
-                    "Generated problem has some validation errors:\n{}",
-                    FormatError::format_many(errors.as_slice(), "\n")
-                );
-            };
-
+pub fn run_generate(matches: &ArgMatches) -> Result<(), String> {
+    match generate_problem_from_args(matches) {
+        Ok((problem, input_format)) => {
             let out_result = matches.value_of(OUT_RESULT_ARG_NAME).map(|path| create_file(path, "out result"));
             let out_buffer = create_write_buffer(out_result);
 
-            let result = match input_format {
-                "pragmatic" => serialize_problem(out_buffer, &problem),
-                "hre" => serialize_hre_problem(out_buffer, &problem),
-                _ => {
-                    eprintln!("Unknown output format: '{}'", input_format);
-                    process::exit(1);
-                }
-            };
-
-            if let Err(err) = result {
-                eprintln!("Cannot serialize result problem: '{}'", err);
-                process::exit(1);
-            } else {
-                println!("Problem has been generated");
+            match input_format.as_str() {
+                "pragmatic" => serialize_problem(out_buffer, &problem)
+                    .map_err(|err| format!("cannot serialize as pragmatic problem: '{}'", err)),
+                "hre" => serialize_hre_problem(out_buffer, &problem)
+                    .map_err(|err| format!("cannot serialize as hre problem: '{}'", err)),
+                _ => Err(format!("unknown output format: '{}'", input_format)),
             }
         }
-        Err(err) => {
-            eprintln!("Cannot generate problem: '{}'", err);
-            process::exit(1);
-        }
+        Err(err) => Err(format!("cannot generate problem: '{}'", err)),
     }
 }
+
+fn generate_problem_from_args(matches: &ArgMatches) -> Result<(Problem, String), String> {
+    let input_format = matches.value_of(FORMAT_ARG_NAME).unwrap();
+
+    let input_files = matches
+        .values_of(PROTOTYPES_ARG_NAME)
+        .map(|paths: Values| paths.map(|path| BufReader::new(open_file(path, "input"))).collect::<Vec<_>>());
+
+    let locations_file = matches.value_of(LOCATIONS_ARG_NAME).map(|path| BufReader::new(open_file(path, "locations")));
+
+    let jobs_size = parse_int_value::<usize>(matches, JOBS_SIZE_ARG_NAME, "jobs size")?.unwrap();
+    let vehicles_size = parse_int_value::<usize>(matches, VEHICLES_SIZE_ARG_NAME, "vehicles size")?.unwrap();
+    let area_size = parse_float_value::<f64>(matches, AREA_SIZE_ARG_NAME, "area size")?;
+
+    generate_problem(input_format, input_files, locations_file, jobs_size, vehicles_size, area_size).and_then(
+        |problem| {
+            ValidationContext::new(&problem, None)
+                .validate()
+                .map_err(|errors| {
+                    format!(
+                        "generated problem has some validation errors:\n{}",
+                        FormatError::format_many(errors.as_slice(), "\n")
+                    )
+                })
+                .map(|_| (problem, input_format.to_owned()))
+        },
+    )
+}
