@@ -7,10 +7,8 @@ use super::*;
 use std::collections::HashMap;
 use std::fs::File;
 use std::io::{BufReader, BufWriter, Write};
-use std::process;
 use std::sync::Arc;
 use vrp_cli::core::solver::population::{get_default_population, Population};
-use vrp_cli::extensions::check::check_pragmatic_solution;
 use vrp_cli::extensions::solve::config::create_builder_from_config_file;
 use vrp_cli::{get_errors_serialized, get_locations_serialized};
 use vrp_core::models::{Problem, Solution};
@@ -177,7 +175,9 @@ pub fn get_solve_app<'a, 'b>() -> App<'a, 'b> {
         )
         .arg(
             Arg::with_name(MIN_CV_ARG_NAME)
-                .help("Specifies variation coefficient termination criteria in form \"sample_size,threshold\"")
+                .help(
+                    "Specifies variation coefficient termination criteria in form \"sample_size,threshold,is_global\"",
+                )
                 .short("v")
                 .long(MIN_CV_ARG_NAME)
                 .required(false)
@@ -281,8 +281,11 @@ pub fn get_solve_app<'a, 'b>() -> App<'a, 'b> {
 }
 
 /// Runs solver commands.
-pub fn run_solve(matches: &ArgMatches, out_writer_func: fn(Option<File>) -> BufWriter<Box<dyn Write>>) {
-    let environment = get_environment(matches);
+pub fn run_solve(
+    matches: &ArgMatches,
+    out_writer_func: fn(Option<File>) -> BufWriter<Box<dyn Write>>,
+) -> Result<(), String> {
+    let environment = get_environment(matches)?;
 
     let formats = get_formats(environment.random.clone());
 
@@ -292,8 +295,8 @@ pub fn run_solve(matches: &ArgMatches, out_writer_func: fn(Option<File>) -> BufW
     let problem_file = open_file(problem_path, "problem");
 
     // optional
-    let max_generations = parse_int_value::<usize>(matches, GENERATIONS_ARG_NAME, "max generations");
-    let max_time = parse_int_value::<usize>(matches, TIME_ARG_NAME, "max time");
+    let max_generations = parse_int_value::<usize>(matches, GENERATIONS_ARG_NAME, "max generations")?;
+    let max_time = parse_int_value::<usize>(matches, TIME_ARG_NAME, "max time")?;
     let telemetry = Telemetry::new(if matches.is_present(LOG_ARG_NAME) {
         TelemetryMode::OnlyLogging {
             logger: Arc::new(|msg| println!("{}", msg)),
@@ -306,9 +309,9 @@ pub fn run_solve(matches: &ArgMatches, out_writer_func: fn(Option<File>) -> BufW
     });
     let is_check_requested = matches.is_present(CHECK_ARG_NAME);
 
-    let min_cv = get_cv(matches);
+    let min_cv = get_cv(matches)?;
     let init_solution = matches.value_of(INIT_SOLUTION_ARG_NAME).map(|path| open_file(path, "init solution"));
-    let init_size = get_init_size(matches);
+    let init_size = get_init_size(matches)?;
     let config = matches.value_of(CONFIG_ARG_NAME).map(|path| open_file(path, "config"));
     let matrix_files = get_matrix_files(matches);
     let out_result = matches.value_of(OUT_RESULT_ARG_NAME).map(|path| create_file(path, "out solution"));
@@ -322,31 +325,22 @@ pub fn run_solve(matches: &ArgMatches, out_writer_func: fn(Option<File>) -> BufW
             let geo_buffer = out_geojson.map(|geojson| create_write_buffer(Some(geojson)));
 
             if is_get_locations_set {
-                locations_writer.0(problem_file, out_buffer).unwrap_or_else(|err| {
-                    eprintln!("cannot get locations '{}'", err);
-                    process::exit(1);
-                });
+                locations_writer.0(problem_file, out_buffer).map_err(|err| format!("cannot get locations '{}'", err))
             } else {
                 match problem_reader.0(problem_file, matrix_files) {
                     Ok(problem) => {
                         let problem = Arc::new(problem);
-                        let solutions = init_solution.map_or_else(Vec::new, |file| {
-                            init_reader.0(file, problem.clone())
-                                .map_err(|err| {
-                                    eprintln!("cannot read initial solution '{}'", err);
-                                    process::exit(1);
-                                })
-                                .map(|solution| vec![solution])
-                                .unwrap()
-                        });
+                        let solutions = init_solution
+                            .map(|file| {
+                                init_reader.0(file, problem.clone())
+                                    .map_err(|err| format!("cannot read initial solution '{}'", err))
+                                    .map(|solution| vec![solution])
+                            })
+                            .unwrap_or_else(|| Ok(Vec::new()))?;
 
                         let builder = if let Some(config) = config {
-                            create_builder_from_config_file(problem.clone(), BufReader::new(config)).unwrap_or_else(
-                                |err| {
-                                    eprintln!("cannot read config: '{}'", err);
-                                    process::exit(1);
-                                },
-                            )
+                            create_builder_from_config_file(problem.clone(), BufReader::new(config))
+                                .map_err(|err| format!("cannot read config: '{}'", err))?
                         } else {
                             Builder::new(problem.clone(), environment.clone())
                                 .with_telemetry(telemetry)
@@ -354,63 +348,65 @@ pub fn run_solve(matches: &ArgMatches, out_writer_func: fn(Option<File>) -> BufW
                                 .with_max_time(max_time)
                                 .with_min_cv(min_cv)
                                 .with_population(get_population(mode, problem.clone(), environment.clone()))
-                                .with_hyper(get_heuristic(matches, problem.clone(), environment))
+                                .with_hyper(get_heuristic(matches, problem.clone(), environment)?)
                         };
 
                         let (solution, _, metrics) = builder
                             .with_init_solutions(solutions, init_size)
                             .build()
                             .and_then(|solver| solver.solve())
-                            .unwrap_or_else(|err| {
-                                eprintln!("cannot find any solution: '{}'", err);
-                                process::exit(1);
-                            });
+                            .map_err(|err| format!("cannot find any solution: '{}'", err))?;
 
                         solution_writer.0(&problem, solution, metrics, out_buffer, geo_buffer).unwrap();
 
                         if is_check_requested {
-                            check_solution(matches);
+                            check_pragmatic_solution_with_args(matches)?;
+                            println!("solution feasibility check is completed successfully");
                         }
+
+                        Ok(())
                     }
                     Err(error) => {
-                        eprintln!("cannot read {} problem from '{}': '{}'", problem_format, problem_path, error);
-                        process::exit(1);
+                        Err(format!("cannot read {} problem from '{}': '{}'", problem_format, problem_path, error))
                     }
-                };
+                }
             }
         }
-        None => {
-            eprintln!("unknown format: '{}'", problem_format);
-            process::exit(1);
-        }
+        None => Err(format!("unknown format: '{}'", problem_format)),
     }
 }
 
-fn get_cv(matches: &ArgMatches) -> Option<(usize, f64)> {
-    matches.value_of(MIN_CV_ARG_NAME).map(|arg| {
-        if let [sample, threshold] =
-            arg.split(',').filter_map(|line| line.parse::<f64>().ok()).collect::<Vec<_>>().as_slice()
-        {
-            (*sample as usize, *threshold)
-        } else {
-            eprintln!("cannot parse min_cv parameter");
-            process::exit(1);
-        }
-    })
+fn get_cv(matches: &ArgMatches) -> Result<Option<(usize, f64, bool)>, String> {
+    let err_result = Err("cannot parse min_cv parameter".to_string());
+    matches
+        .value_of(MIN_CV_ARG_NAME)
+        .map(|arg| match arg.split(',').collect::<Vec<_>>().as_slice() {
+            [sample, threshold, is_global] => {
+                match (sample.parse::<usize>(), threshold.parse::<f64>(), is_global.parse::<bool>()) {
+                    (Ok(sample), Ok(threshold), Ok(is_global)) => Ok(Some((sample, threshold, is_global))),
+                    _ => err_result,
+                }
+            }
+            _ => err_result,
+        })
+        .unwrap_or(Ok(None))
 }
 
-fn get_init_size(matches: &ArgMatches) -> Option<usize> {
-    matches.value_of(INIT_SIZE_ARG_NAME).map(|size| {
-        if let Some(value) = size.parse::<usize>().ok().and_then(|value| if value < 1 { None } else { Some(value) }) {
-            value
-        } else {
-            eprintln!("init size must be a value than 0, got '{}'", size);
-            process::exit(1);
-        }
-    })
+fn get_init_size(matches: &ArgMatches) -> Result<Option<usize>, String> {
+    matches
+        .value_of(INIT_SIZE_ARG_NAME)
+        .map(|size| {
+            if let Some(value) = size.parse::<usize>().ok().and_then(|value| if value < 1 { None } else { Some(value) })
+            {
+                Ok(Some(value))
+            } else {
+                Err(format!("init size must be an integer bigger than 0, got '{}'", size))
+            }
+        })
+        .unwrap_or(Ok(None))
 }
 
-fn get_environment(matches: &ArgMatches) -> Arc<Environment> {
+fn get_environment(matches: &ArgMatches) -> Result<Arc<Environment>, String> {
     matches
         .value_of(PARALELLISM_ARG_NAME)
         .map(|arg| {
@@ -418,13 +414,12 @@ fn get_environment(matches: &ArgMatches) -> Arc<Environment> {
                 arg.split(',').filter_map(|line| line.parse::<usize>().ok()).collect::<Vec<_>>().as_slice()
             {
                 let parallelism = Parallelism::new(*num_thread_pools, *threads_per_pool);
-                Arc::new(Environment::new(Arc::new(DefaultRandom::default()), parallelism))
+                Ok(Arc::new(Environment::new(Arc::new(DefaultRandom::default()), parallelism)))
             } else {
-                eprintln!("cannot parse parallelism parameter");
-                process::exit(1);
+                Err("cannot parse parallelism parameter".to_string())
             }
         })
-        .unwrap_or_else(|| Arc::new(Environment::default()))
+        .unwrap_or_else(|| Ok(Arc::new(Environment::default())))
 }
 
 fn get_matrix_files(matches: &ArgMatches) -> Option<Vec<File>> {
@@ -453,38 +448,15 @@ fn get_heuristic(
     matches: &ArgMatches,
     problem: Arc<Problem>,
     environment: Arc<Environment>,
-) -> Box<dyn HyperHeuristic + Send + Sync> {
+) -> Result<Box<dyn HyperHeuristic + Send + Sync>, String> {
     match matches.value_of(HEURISTIC_ARG_NAME) {
-        Some("dynamic") => Box::new(DynamicSelective::new_with_defaults(problem, environment)),
-        Some("static") => Box::new(StaticSelective::new_with_defaults(problem, environment)),
-        Some(name) if name != "default" => {
-            eprintln!("unknown heuristic type name: '{}'", name);
-            process::exit(1);
-        }
-        _ => Box::new(MultiSelective::new_with_defaults(problem, environment)),
+        Some("dynamic") => Ok(Box::new(DynamicSelective::new_with_defaults(problem, environment))),
+        Some("static") => Ok(Box::new(StaticSelective::new_with_defaults(problem, environment))),
+        Some(name) if name != "default" => Err(format!("unknown heuristic type name: '{}'", name)),
+        _ => Ok(Box::new(MultiSelective::new_with_defaults(problem, environment))),
     }
 }
 
-fn check_solution(matches: &ArgMatches) {
-    let problem_file = matches
-        .value_of(PROBLEM_ARG_NAME)
-        .map(|path| BufReader::new(open_file(path, "problem")))
-        .expect("cannot read problem");
-    let solution_file = matches
-        .value_of(OUT_RESULT_ARG_NAME)
-        .map(|path| BufReader::new(open_file(path, "solution")))
-        .expect("cannot read solution");
-
-    let matrix_files = matches
-        .values_of(MATRIX_ARG_NAME)
-        .map(|paths: Values| paths.map(|path| BufReader::new(open_file(path, "routing matrix"))).collect());
-
-    let result = check_pragmatic_solution(problem_file, solution_file, matrix_files);
-
-    if let Err(err) = result {
-        eprintln!("checker found {} errors:\n{}", err.len(), err.join("\n"));
-        process::exit(1);
-    } else {
-        println!("solution feasibility check is completed successfully");
-    }
+fn check_pragmatic_solution_with_args(matches: &ArgMatches) -> Result<(), String> {
+    check_solution(matches, "pragmatic", PROBLEM_ARG_NAME, OUT_RESULT_ARG_NAME, MATRIX_ARG_NAME)
 }
