@@ -83,7 +83,7 @@ pub enum PopulationType {
         learning_rate: Option<f64>,
         /// A rebalance memory. Default is 500.
         rebalance_memory: Option<usize>,
-        /// A rebalance count. Default is 10.
+        /// A rebalance count. Default is 2.
         rebalance_count: Option<usize>,
         /// An exploration phase ratio. Default is 0.9.
         exploration_ratio: Option<f64>,
@@ -287,6 +287,9 @@ pub enum RecreateMethod {
     /// Nearest neighbour method.
     #[serde(rename(deserialize = "nearest"))]
     Nearest { weight: usize },
+    #[serde(rename(deserialize = "skip-random"))]
+    /// Insertion with skip random method.
+    SkipRandom { weight: usize },
     /// Farthest insertion method.
     #[serde(rename(deserialize = "farthest"))]
     Farthest { weight: usize },
@@ -326,9 +329,11 @@ pub struct TerminationConfig {
 }
 
 #[derive(Clone, Deserialize, Debug)]
+#[serde(rename_all = "camelCase")]
 pub struct VariationConfig {
     sample: usize,
     cv: f64,
+    is_global: bool,
 }
 
 /// A telemetry config.
@@ -514,7 +519,7 @@ fn configure_from_termination(mut builder: Builder, termination_config: &Option<
     if let Some(config) = termination_config {
         builder = builder.with_max_time(config.max_time);
         builder = builder.with_max_generations(config.max_generations);
-        builder = builder.with_min_cv(config.variation.as_ref().map(|v| (v.sample, v.cv)));
+        builder = builder.with_min_cv(config.variation.as_ref().map(|v| (v.sample, v.cv, v.is_global)));
     }
 
     builder
@@ -531,6 +536,7 @@ fn create_recreate_method(
         RecreateMethod::Blinks { weight } => {
             (Arc::new(RecreateWithBlinks::<SingleDimLoad>::new_with_defaults(environment.random.clone())), *weight)
         }
+        RecreateMethod::SkipRandom { weight } => (Arc::new(RecreateWithSkipRandom::default()), *weight),
         RecreateMethod::Gaps { weight, min } => (Arc::new(RecreateWithGaps::new(*min)), *weight),
         RecreateMethod::Nearest { weight } => (Arc::new(RecreateWithNearestNeighbor::default()), *weight),
         RecreateMethod::Regret { weight, start, end } => (Arc::new(RecreateWithRegret::new(*start, *end)), *weight),
